package cz.jzitnik.game;

import cz.jzitnik.tui.ScreenMovingCalculationProvider;
import cz.jzitnik.tui.ScreenRenderer;
import lombok.AllArgsConstructor;
import org.jline.terminal.MouseEvent;
import org.jline.terminal.Terminal;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@AllArgsConstructor
public class MouseHandler {
    private Game game;
    private Terminal terminal;
    private ScreenRenderer screenRenderer;

    public void handle(MouseEvent mouseEvent) {
        if (mouseEvent.getButton() == MouseEvent.Button.Button1 && mouseEvent.getType() == MouseEvent.Type.Pressed) {
            leftClick(mouseEvent);
            return;
        }

        if (mouseEvent.getType() == MouseEvent.Type.Moved) {
            move(mouseEvent);
        }
    }

    private void leftClick(MouseEvent mouseEvent) {
        int mouseX = mouseEvent.getX();
        int mouseY = mouseEvent.getY();

        int[] cords = game.getPlayerCords();
        if (cords == null) return;

        int playerX = cords[0];
        int playerY = cords[1];

        int[] data = ScreenMovingCalculationProvider.calculate(playerX, playerY, terminal.getHeight(), terminal.getWidth(), game.getWorld()[0].length, game.getWorld().length);

        int startX = data[0];
        int endX = data[1];
        int startY = data[2];
        int endY = data[3];

        int blockX = startX + (mouseX / 50);  // 50 chars wide per sprite
        int blockY = startY + (mouseY / 25);  // 25 lines high per sprite

        if (blockY == playerX && blockY == playerY) {
            return;
        }

        if (blockY >= startY && blockY < endY && blockX >= startX && blockX < endX) {
            List<Block> blocks = game.getWorld()[blockY][blockX];
            System.out.println(blockX);
            System.out.println(blockY);
            if (!blocks.isEmpty()) {
                game.mine(screenRenderer, blockX, blockY);
            }
        }
    }

    private void move(MouseEvent mouseEvent) {
        int mouseX = mouseEvent.getX();
        int mouseY = mouseEvent.getY();

        int[] cords = game.getPlayerCords();
        if (cords == null) return;

        int playerX = cords[0];
        int playerY = cords[1];

        int[] data = ScreenMovingCalculationProvider.calculate(playerX, playerY, terminal.getHeight(), terminal.getWidth(), game.getWorld()[0].length, game.getWorld().length);

        int startX = data[0];
        int endX = data[1];
        int startY = data[2];
        int endY = data[3];

        int blockX = startX + (mouseX / 50);  // 50 chars wide per sprite
        int blockY = startY + (mouseY / 25);  // 25 lines high per sprite

        if (blockY == playerX && blockY == playerY) {
            return;
        }

        List<Block> blocks = game.getWorld()[blockY][blockX];

        if (blockY >= startY && blockY < endY && blockX >= startX && blockX < endX && !blocks.stream().allMatch(block -> block.getBlockId().equals("air"))) {
            List<Integer> list = new ArrayList<>();
            list.add(blockX);
            list.add(blockY);
            screenRenderer.setSelectedBlock(Optional.of(list));
        } else {
            screenRenderer.setSelectedBlock(Optional.empty());
        }

        screenRenderer.render(game.getWorld());
    }
}
