package cz.jzitnik.tui;

import cz.jzitnik.game.Block;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.Setter;
import org.jline.terminal.Terminal;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@RequiredArgsConstructor
public class ScreenRenderer {
    private SpriteList spriteList;
    private Terminal terminal;

    public ScreenRenderer(SpriteList spriteList, Terminal terminal) {
        this.spriteList = spriteList;
        this.terminal = terminal;
    }

    @Getter
    @Setter
    private Optional<List<Integer>> selectedBlock = Optional.empty();

    private int[] getPlayerCords(List<Block>[][] world) {
        for (int i = 0; i < world.length; i++) {
            for (int j = 0; j < world[i].length; j++) {
                if (world[i][j].stream().anyMatch(x -> x.getBlockId().equals("steve"))) {
                    return new int[]{j, i};
                }
            }
        }
        return null;
    }

    public void render(List<Block>[][] world) {
        StringBuilder main = new StringBuilder();
        main.append("\033[H\033[2J");

        int[] cords = getPlayerCords(world);
        if (cords == null) return;

        int playerX = cords[0];
        int playerY = cords[1];

        int terminalWidth = terminal.getWidth();
        int terminalHeight = terminal.getHeight();

        int[] data = ScreenMovingCalculationProvider.calculate(playerX, playerY, terminalHeight, terminalWidth, world[0].length, world.length);

        // Calculate visible area boundaries
        int startX = data[0];
        int endX = data[1];
        int startY = data[2];
        int endY = data[3];

        int visibleWidth = endX - startX;
        int visibleHeight = endY - startY;

        // If the width is odd, reduce viewXRadius by 1
        if (visibleWidth % 2 != 0) {
            endX = Math.max(startX, endX - 1);
        }
        if (visibleHeight % 2 != 0) {
            endY = Math.max(startY, endY - 1);
        }

        StringBuilder[] lines = new StringBuilder[(endY - startY) * 26];
        for (int i = 0; i < lines.length; i++) {
            lines[i] = new StringBuilder();
        }

        int counter = 0;
        for (int y = startY; y < endY; y++) {
            for (int x = startX; x < endX; x++) {
                List<Block> blocks = world[y][x];
                List<String> sprites = new ArrayList<>(blocks.stream()
                        .map(block -> spriteList.getSprite(block.getSprite()).getSprite()).toList());

                if (selectedBlock.isPresent() && selectedBlock.get().get(0) == x && selectedBlock.get().get(1) == y) {
                    StringBuilder stringBuilder = new StringBuilder();

                    for (int i = 0; i < 50; i++) {
                        stringBuilder.append("\033[38;5;231;48;5;231m▓");
                    }
                    stringBuilder.append("\n");

                    for (int i = 0; i < 23; i++) {
                        stringBuilder.append("\033[38;5;231;48;5;231m▓");
                        for (int j = 0; j < 48; j++) {
                            stringBuilder.append("\033[0m ");
                        }
                        stringBuilder.append("\033[38;5;231;48;5;231m▓");
                        stringBuilder.append("\n");
                    }

                    for (int i = 0; i < 50; i++) {
                        stringBuilder.append("\033[38;5;231;48;5;231m▓");
                    }
                    stringBuilder.append("\n");

                    sprites.add(stringBuilder.toString());
                }

                String sprite = SpriteCombiner.combineSprites(sprites.toArray(String[]::new));


                String[] spriteLines = sprite.split("\n");

                for (int i = 0; i < spriteLines.length; i++) {
                    lines[counter * 25 + i].append(spriteLines[i]);
                }
            }
            counter++;
        }

        for (StringBuilder line : lines) {
            main.append(line.toString());
            main.append("\n");
        }

        System.out.println(main);
    }
}
